@@ -1,15 +1,21 @@
 from __future__ import annotations
 
+import socket
+from collections.abc import Sequence
+
 import pytest
 from _pytest.logging import LogCaptureFixture
 from _pytest.pytester import Pytester
 
-from anyio import get_all_backends
+from anyio import get_all_backends, get_available_backends
+from anyio.pytest_plugin import FreePortFactory
 
-pytestmark = pytest.mark.filterwarnings(
-    "ignore:The TerminalReporter.writer attribute is deprecated"
-    ":pytest.PytestDeprecationWarning:"
-)
+pytestmark = [
+    pytest.mark.filterwarnings(
+        "ignore:The TerminalReporter.writer attribute is deprecated"
+        ":pytest.PytestDeprecationWarning:"
+    )
+]
 
 pytest_args = "-v", "-p", "anyio", "-p", "no:asyncio", "-p", "no:trio"
 
@@ -18,10 +24,10 @@ def test_plugin(testdir: Pytester) -> None:
     testdir.makeconftest(
         """
         from contextvars import ContextVar
-        import sniffio
         import pytest
 
         from anyio import sleep
+        from anyio._core._eventloop import current_async_library
 
         var = ContextVar("var")
 
@@ -29,7 +35,7 @@ def test_plugin(testdir: Pytester) -> None:
         @pytest.fixture
         async def async_fixture():
             await sleep(0)
-            return sniffio.current_async_library()
+            return current_async_library()
 
 
         @pytest.fixture
@@ -49,10 +55,10 @@ def test_plugin(testdir: Pytester) -> None:
     testdir.makepyfile(
         """
         import pytest
-        import sniffio
         from hypothesis import strategies, given
 
         from anyio import get_all_backends, sleep
+        from anyio._core._eventloop import current_async_library
 
 
         @pytest.mark.anyio
@@ -83,7 +89,7 @@ def test_plugin(testdir: Pytester) -> None:
 
     result = testdir.runpytest(*pytest_args)
     result.assert_outcomes(
-        passed=4 * len(get_all_backends()), skipped=len(get_all_backends())
+        passed=4 * len(get_available_backends()), skipped=len(get_available_backends())
     )
 
 
@@ -210,7 +216,6 @@ def test_autouse_async_fixture(testdir: Pytester) -> None:
         """
         import pytest
 
-        import sniffio
         from anyio import get_all_backends, sleep
 
 
@@ -221,7 +226,7 @@ def test_autouse_async_fixture(testdir: Pytester) -> None:
     )
 
     result = testdir.runpytest_subprocess(*pytest_args)
-    result.assert_outcomes(passed=len(get_all_backends()))
+    result.assert_outcomes(passed=len(get_available_backends()))
 
 
 def test_cancel_scope_in_asyncgen_fixture(testdir: Pytester) -> None:
@@ -248,7 +253,7 @@ def test_cancel_scope_in_asyncgen_fixture(testdir: Pytester) -> None:
     )
 
     result = testdir.runpytest_subprocess(*pytest_args)
-    result.assert_outcomes(passed=len(get_all_backends()))
+    result.assert_outcomes(passed=len(get_available_backends()))
 
 
 def test_module_scoped_task_group_fixture(testdir: Pytester) -> None:
@@ -260,11 +265,11 @@ def test_module_scoped_task_group_fixture(testdir: Pytester) -> None:
             CancelScope,
             create_memory_object_stream,
             create_task_group,
-            get_all_backends,
+            get_available_backends,
         )
 
 
-        @pytest.fixture(scope="module", params=get_all_backends())
+        @pytest.fixture(scope="module", params=get_available_backends())
         def anyio_backend():
             return 'asyncio'
 
@@ -305,7 +310,7 @@ def test_module_scoped_task_group_fixture(testdir: Pytester) -> None:
     )
 
     result = testdir.runpytest_subprocess(*pytest_args)
-    result.assert_outcomes(passed=len(get_all_backends()))
+    result.assert_outcomes(passed=len(get_available_backends()))
 
 
 def test_async_fixture_teardown_after_sync_test(testdir: Pytester) -> None:
@@ -375,7 +380,7 @@ def test_hypothesis_module_mark(testdir: Pytester) -> None:
 
     result = testdir.runpytest(*pytest_args)
     result.assert_outcomes(
-        passed=len(get_all_backends()) + 1, xfailed=len(get_all_backends())
+        passed=len(get_available_backends()) + 1, xfailed=len(get_available_backends())
     )
 
 
@@ -416,11 +421,12 @@ def test_hypothesis_function_mark(testdir: Pytester) -> None:
 
     result = testdir.runpytest(*pytest_args)
     result.assert_outcomes(
-        passed=2 * len(get_all_backends()), xfailed=2 * len(get_all_backends())
+        passed=2 * len(get_available_backends()),
+        xfailed=2 * len(get_available_backends()),
     )
 
 
-@pytest.mark.parametrize("anyio_backend", get_all_backends(), indirect=True)
+@pytest.mark.parametrize("anyio_backend", get_available_backends(), indirect=True)
 def test_debugger_exit_in_taskgroup(testdir: Pytester, anyio_backend_name: str) -> None:
     testdir.makepyfile(
         f"""
@@ -468,3 +474,205 @@ def test_keyboardinterrupt_during_test(
     )
 
     testdir.runpytest_subprocess(*pytest_args, timeout=3)
+
+
+def test_async_fixture_in_test_class(testdir: Pytester) -> None:
+    # Regression test for #633
+    testdir.makepyfile(
+        """
+        import pytest
+
+
+        class TestAsyncFixtureMethod:
+            is_same_instance = False
+
+            @pytest.fixture(autouse=True)
+            async def async_fixture_method(self):
+                self.is_same_instance = True
+
+            @pytest.mark.anyio
+            async def test_async_fixture_method(self):
+                assert self.is_same_instance
+        """
+    )
+
+    result = testdir.runpytest_subprocess(*pytest_args)
+    result.assert_outcomes(passed=len(get_available_backends()))
+
+
+def test_asyncgen_fixture_in_test_class(testdir: Pytester) -> None:
+    # Regression test for #633
+    testdir.makepyfile(
+        """
+        import pytest
+
+
+        class TestAsyncFixtureMethod:
+            is_same_instance = False
+
+            @pytest.fixture(autouse=True)
+            async def async_fixture_method(self):
+                self.is_same_instance = True
+                yield
+
+            @pytest.mark.anyio
+            async def test_async_fixture_method(self):
+                assert self.is_same_instance
+        """
+    )
+
+    result = testdir.runpytest_subprocess(*pytest_args)
+    result.assert_outcomes(passed=len(get_available_backends()))
+
+
+def test_async_fixture_params(testdir: Pytester) -> None:
+    testdir.makepyfile(
+        """
+        import inspect
+        import pytest
+
+        @pytest.fixture(params=[1, 2])
+        async def fixt(request):
+            return request.param
+
+        @pytest.mark.anyio
+        async def test_params(fixt):
+            assert fixt in (1, 2)
+        """
+    )
+
+    result = testdir.runpytest(*pytest_args)
+    result.assert_outcomes(passed=len(get_available_backends()) * 2)
+
+
+def test_auto_mode(testdir: Pytester) -> None:
+    testdir.makepyprojecttoml(
+        """
+        [tool.pytest.ini_options]
+        anyio_mode = "auto"
+        """
+    )
+    testdir.makepyfile(
+        """
+        import inspect
+        import pytest
+
+        @pytest.fixture
+        async def fixt(request):
+            return 1
+
+        async def test_params(fixt):
+            assert fixt == 1
+        """
+    )
+
+    result = testdir.runpytest(*pytest_args)
+    result.assert_outcomes(passed=len(get_available_backends()))
+
+
+def test_auto_mode_conflict_warning(testdir: Pytester) -> None:
+    testdir.makepyprojecttoml(
+        """
+        [tool.pytest.ini_options]
+        anyio_mode = "auto"
+        asyncio_mode = "auto"
+        asyncio_default_fixture_loop_scope = "function"
+        """
+    )
+    testdir.makeconftest(
+        """
+        import pytest
+
+        class FakeAsyncioPlugin:
+            def pytest_addoption(self, parser, pluginmanager):
+                parser.addini(
+                    "asyncio_mode",
+                    default="strict",
+                    type="string",
+                    help="dummy asyncio plugin"
+                )
+
+        def pytest_addhooks(pluginmanager):
+            if not pluginmanager.has_plugin("asyncio"):
+                pluginmanager.register(FakeAsyncioPlugin(), "asyncio")
+        """
+    )
+    testdir.makepyfile(
+        """
+        import inspect
+        import pytest
+
+        @pytest.fixture
+        async def fixt(request):
+            return 1
+
+        async def test_params(fixt):
+            assert fixt == 1
+        """
+    )
+
+    result = testdir.runpytest("-p", "anyio")
+    result.assert_outcomes(passed=len(get_available_backends()))
+    assert (
+        "PytestConfigWarning: AnyIO auto mode has been enabled together with "
+        "pytest-asyncio auto mode. This may cause unexpected behavior."
+    ) in result.stdout.str()
+
+
+class TestFreePortFactory:
+    @pytest.fixture(scope="class")
+    def families(self) -> Sequence[tuple[socket.AddressFamily, str]]:
+        from .test_sockets import has_ipv6
+
+        families: list[tuple[socket.AddressFamily, str]] = [
+            (socket.AF_INET, "127.0.0.1")
+        ]
+        if has_ipv6:
+            families.append((socket.AF_INET6, "::1"))
+
+        return families
+
+    async def test_tcp_factory(
+        self,
+        families: Sequence[tuple[socket.AddressFamily, str]],
+        free_tcp_port_factory: FreePortFactory,
+    ) -> None:
+        generated_ports = {free_tcp_port_factory() for _ in range(5)}
+        assert all(isinstance(port, int) for port in generated_ports)
+        assert len(generated_ports) == 5
+        for port in generated_ports:
+            for family, addr in families:
+                with socket.socket(family, socket.SOCK_STREAM) as sock:
+                    try:
+                        sock.bind((addr, port))
+                    except OSError:
+                        pass
+
+    async def test_udp_factory(
+        self,
+        families: Sequence[tuple[socket.AddressFamily, str]],
+        free_udp_port_factory: FreePortFactory,
+    ) -> None:
+        generated_ports = {free_udp_port_factory() for _ in range(5)}
+        assert all(isinstance(port, int) for port in generated_ports)
+        assert len(generated_ports) == 5
+        for port in generated_ports:
+            for family, addr in families:
+                with socket.socket(family, socket.SOCK_DGRAM) as sock:
+                    sock.bind((addr, port))
+
+    async def test_free_tcp_port(
+        self, families: Sequence[tuple[socket.AddressFamily, str]], free_tcp_port: int
+    ) -> None:
+        assert isinstance(free_tcp_port, int)
+        for family, addr in families:
+            with socket.socket(family, socket.SOCK_STREAM) as sock:
+                sock.bind((addr, free_tcp_port))
+
+    async def test_free_udp_port(
+        self, families: Sequence[tuple[socket.AddressFamily, str]], free_udp_port: int
+    ) -> None:
+        assert isinstance(free_udp_port, int)
+        for family, addr in families:
+            with socket.socket(family, socket.SOCK_DGRAM) as sock:
+                sock.bind((addr, free_udp_port))
