@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import asyncio
+import sys
 from typing import Any
 
 import pytest
@@ -24,8 +25,6 @@ from anyio.lowlevel import checkpoint
 
 from .conftest import asyncio_params
 
-pytestmark = pytest.mark.anyio
-
 
 class TestLock:
     async def test_contextmanager(self) -> None:
@@ -401,6 +400,13 @@ class TestCondition:
         assert task_started
         assert not notified
 
+    async def test_wait_no_lock(self) -> None:
+        condition = Condition()
+        with pytest.raises(
+            RuntimeError, match="The current task is not holding the underlying lock"
+        ):
+            await condition.wait()
+
     async def test_statistics(self) -> None:
         async def waiter() -> None:
             async with condition:
@@ -445,6 +451,30 @@ class TestCondition:
             backend_options=anyio_backend_options,
         )
 
+    async def test_wait_for(self) -> None:
+        result = None
+
+        async def waiter() -> None:
+            nonlocal result
+            async with condition:
+                result = await condition.wait_for(lambda: value)
+
+        value = None
+        condition = Condition()
+        async with create_task_group() as tg:
+            tg.start_soon(waiter)
+            await wait_all_tasks_blocked()
+            async with condition:
+                condition.notify_all()
+
+            await wait_all_tasks_blocked()
+            assert result is None
+            value = "foo"
+            async with condition:
+                condition.notify_all()
+
+        assert result == "foo"
+
 
 class TestSemaphore:
     async def test_contextmanager(self) -> None:
@@ -637,8 +667,25 @@ class TestCapacityLimiter:
             "total_tokens must be an int or math.inf"
         )
 
-    async def test_bad_init_value(self) -> None:
-        pytest.raises(ValueError, CapacityLimiter, 0).match("total_tokens must be >= 1")
+    async def test_bad_init_value(self, anyio_backend_name: str) -> None:
+        # TODO: Remove this once Python 3.9 is dropped
+        if sys.version_info < (3, 10) and anyio_backend_name == "trio":
+            bad_value = 0
+            min_value = 1
+        else:
+            bad_value = -1
+            min_value = 0
+
+        pytest.raises(ValueError, CapacityLimiter, bad_value).match(
+            f"total_tokens must be >= {min_value}"
+        )
+
+    async def test_zero_tokens(self, anyio_backend_name: str) -> None:
+        if sys.version_info < (3, 10) and anyio_backend_name == "trio":
+            pytest.skip("Trio does not support zero-capacity limiters on Python 3.9")
+
+        limiter = CapacityLimiter(0)
+        assert limiter.total_tokens == 0
 
     async def test_borrow(self) -> None:
         limiter = CapacityLimiter(2)
